package org.stepic.droid.jsonHelpers.adapters

import com.google.gson.GsonBuilder
import org.junit.Assert.assertEquals
import org.junit.Test
import org.stepik.android.domain.personal_deadlines.model.DeadlinesWrapper
import org.stepic.droid.util.toObject
import java.util.*
import java.util.concurrent.Callable
import java.util.concurrent.Executors

class UTCDateAdapterTest {
    @Test
    fun deserializationTest() {
        val gson = GsonBuilder().registerTypeAdapter(Date::class.java, UTCDateAdapter()).create()
        val deadlines = """{"course":67,"deadlines":[{"section":123,"deadline":"2012-04-23T18:25:43.000Z"}]}"""

        val wrapper = deadlines.toObject<DeadlinesWrapper>(gson)
        assertEquals(gson.toJson(wrapper), deadlines)
    }

    @Test
    fun dateDeserializationTest() {
        val dateString = "2018-04-04T09:08:09.000Z"
        val dateAdapter = UTCDateAdapter()

        val date = dateAdapter.stringToDate(dateString)
        assertEquals(1522832889000, date?.time)
    }

    @Test
    fun dateDeserializationThreadSafetyTest() {
        val dateString = "2014-06-19T08:22:37Z"

        val dateAdapter = UTCDateAdapter()
        val executor = Executors.newFixedThreadPool(100)

        val task = Callable {
            dateAdapter.stringToDate(dateString)
        }

        val tasks = (0..100)
            .map { executor.submit(task) }

        executor.shutdown()

        tasks.forEach {
            it.get()
        }
    }
}